from maya.api import OpenMaya
# pylint: enable=no-name-in-module
# noinspection PyUnresolvedReferences
from mocapx.vendor.Qt import QtWidgets, QtCore
from mocapx.ui.widgets import Liner
from mocapx.lib import nodes
from mocapx.lib.utils import get_mobject
from mocapx.lib.uiutils import is_widget_really_visible


class DataSourceAttrPane(QtWidgets.QScrollArea):
    def __init__(self, source_node, parent=None):
        super(DataSourceAttrPane, self).__init__(parent)

        self.node = source_node
        self.attr_dict = dict()

        if nodes.realtime_device_node(self.node):
            self.rd_connection_status = nodes.get_realtime_device_conn_status(self.node)
        else:
            self.rd_connection_status = None

        self.attr_changed_cbid = None
        self.rd_connection_changed_cbid = None
        self.cr_time_change_cbid = None

        self.setWidgetResizable(True)
        self.setFocusPolicy(QtCore.Qt.NoFocus)
        self.setFrameShadow(QtWidgets.QFrame.Plain)

        inner = QtWidgets.QFrame(self)
        self.layout = QtWidgets.QVBoxLayout()
        self.layout.setContentsMargins(0, 0, 0, 0)
        self.layout.setSpacing(3)
        inner.setLayout(self.layout)
        # Set main widget
        self.setWidget(inner)

        self.setFixedHeight(400)

        self.refresh_state()

    def refresh_state(self):
        self.clean_callbacks()
        self.set_callbacks()
        self.update_view(refresh_list=True, force=True)

    def update_view(self, refresh_list=True, force=False):
        if not (is_widget_really_visible(self) or force):
            return
        if not refresh_list:
            if not nodes.node_exists(self.node):
                return
        else:
            # clear all
            item = self.layout.takeAt(0)
            while item:
                if item:
                    widget = item.widget()
                    if widget:
                        self.layout.removeWidget(widget)
                        widget.deleteLater()
                    del item
                    item = self.layout.takeAt(0)
            self.attr_dict.clear()

            # update attributes list
            if nodes.node_exists(self.node):
                attr_list = sorted(nodes.list_attrs(self.node, only_valid=True, userDefined=True))
            else:
                return

            # create new widgets
            for attr in attr_list:
                widget = Liner(label=attr, preview_precision=3)
                widget.setFixedHeight(16)
                widget.valueChanged.connect(self.set_value_handle)
                self.layout.addWidget(widget)
                self.attr_dict[attr] = widget
            self.layout.addStretch()

        # update values
        node_mobject = get_mobject(self.node)
        if node_mobject:
            d_node = OpenMaya.MFnDependencyNode(node_mobject)
            for (attr_1, item_1) in self.attr_dict.items():
                item_1.setValue(d_node.findPlug(attr_1, False).asDouble())

    def set_value_handle(self, widget, value):
        widget.blockSignals(True)
        nodes.set_attr_value(self.node + "." + widget.label(), value=value, force=False, verbose=False)
        widget.blockSignals(False)

    # callbacks
    def status_changed_cb(self, data):
        status = nodes.get_realtime_device_conn_status(self.node)
        if status != self.rd_connection_status:
            self.rd_connection_status = status
            self.refresh_state()

    def attr_changed_cb(self, msg, plug, otherplug, clientData):
        attr_name = plug.partialName(includeNodeName=False)
        if attr_name in ("refreshNode", "refresh", "clipFrameOffset"):
            self.update_view(refresh_list=False)
        else:
            if msg & OpenMaya.MNodeMessage.kAttributeAdded or msg & OpenMaya.MNodeMessage.kAttributeRemoved:
                self.update_view(refresh_list=True)

    def time_changed_cb(self, *args, **kwargs):
        self.update_view(refresh_list=False)

    def clean_callbacks(self):
        if self.rd_connection_changed_cbid:
            OpenMaya.MMessage.removeCallback(self.rd_connection_changed_cbid)
            self.rd_connection_changed_cbid = None
        if self.attr_changed_cbid:
            OpenMaya.MMessage.removeCallback(self.attr_changed_cbid)
            self.attr_changed_cbid = None
        if self.cr_time_change_cbid:
            OpenMaya.MMessage.removeCallback(self.cr_time_change_cbid)
            self.cr_time_change_cbid = None

    def set_callbacks(self):
        node_mobject = get_mobject(self.node)
        if node_mobject:
            self.attr_changed_cbid = OpenMaya.MNodeMessage.addAttributeChangedCallback(
                node_mobject, self.attr_changed_cb)

        if nodes.realtime_device_node(self.node):
            self.rd_connection_changed_cbid = OpenMaya.MUserEventMessage.addUserEventCallback(
                'MCPXRltDvceConnStatusChanged', self.status_changed_cb)
        else:
            self.cr_time_change_cbid = OpenMaya.MDGMessage.addTimeChangeCallback(self.time_changed_cb)
